//Title:       RFTP ConsoleInterface
///Version:    1.0
//Copyright:   1999
//Author:      Mary Grzywa
//Company:     Illinois State University Graduate Project
//Description: Resumable File Transfer Protocol
package rftp;
  
import java.util.*;
import java.util.Vector.*;
import java.util.Date.*;
import java.lang.*;
import java.lang.String.*;
import java.io.*;
import java.net.*;

public class RftpConsoleInterface {

private RftpClient client;
private BufferedReader consoleIn = new BufferedReader(new InputStreamReader(System.in));
private StringBuffer request = new StringBuffer("");
private String serverName = null;
boolean onWindows = true;

public RftpConsoleInterface(String[] args)
{
    String secondCmdInfo = null;
    client = new RftpClient(args);

    File here = new File(".");
    String wd = here.getAbsolutePath();
    int mslast = wd.lastIndexOf("\\");
	  int unixlast = wd.lastIndexOf("/");
   	if (mslast < unixlast)  onWindows = false;
   
    if (args.length < 1)   // no server name provided
    {
	    System.out.print("Please enter the IP address of the server: ");
	    serverName = getConsoleInput();
    }
    else serverName = args[0];
    client.setServerName(serverName);
    System.out.println("IP entered: **" + serverName + "** about to contact server");
    // contact the server
    String contactString = client.contactServer();
    System.out.println(contactString);
    if (contactString.startsWith("1",0))    //server has asked client to wait for 2nd message
	    secondCmdInfo = client.cmdIn();

    if (contactString.startsWith("2",0) || secondCmdInfo.startsWith("2",0) )    //success message
    { 
	while (requestInput()) System.out.println(client.performRequest(request));
    }
    else    //server could not be contacted
    {
        System.out.println(contactString);
    }
}
//*******************************************************************************************
//*******************************************************************************************

public static void main(String[] args) 
{
    RftpConsoleInterface console = new RftpConsoleInterface(args);
}   //end public static void main(String[] args) 

//*******************************************************************************************
//*******************************************************************************************

protected boolean requestInput() 
{   
    System.out.print("Rftp>");
    // put the line read in from the console into the request StringBuffer
    request.setLength(0);
    request.append(getConsoleInput());
    System.out.println(request.toString());

    // convert the command part of the input (first word) to upper case
    String makeUpper = request.toString();
    int endCmd;
    if (makeUpper.indexOf(' ') > 0) endCmd = makeUpper.indexOf(' ');
    	else endCmd = makeUpper.length();
    String CmdInUpperCase = makeUpper.substring(0,endCmd).toUpperCase();
    for (int i = 0; i < endCmd; i++)  
    {
	    char c = CmdInUpperCase.charAt(i);
	    request.setCharAt(i,c);
    }
   // System.out.println("Requested input was:***" + request.toString() + "***");
    if (request.toString().startsWith("QUIT")) return false;
       else return true;
}  //end protected boolean requestInput() 

protected String getConsoleInput()
{
    StringBuffer userInput = new StringBuffer(30);
    try
    {
      char c = (char) System.in.read();
	while (c != '\n')    
	{
	  if (c != '\n') userInput.append(c);
	  c = (char) System.in.read();
	 //System.out.println(userInput.toString());
	}
    } catch (IOException exc) {System.out.println("Read Error: " + exc);}
	// System.out.println("Sending back: " + userInput.toString());

    if (onWindows) return userInput.toString().substring(0,(userInput.length() - 1));
     else   return userInput.toString().substring(0,(userInput.length()));
}  //end protected String getConsoleInput() 

//*******************************************************************************************
//*******************************************************************************************


}
