//Title:       RFTP CurrentDirectory
///Version:    1.0
//Copyright:   1999
//Author:      Mary Grzywa
//Company:     Illinois State University Graduate Project
//Description: Resumable File Transfer Protocol
package rftp;

import java.util.*;
import java.lang.*;
import java.lang.String.*;
import java.io.*;
import java.net.*;

// This class is needed because java does not allow a change in directory.
// In ftp specification, the server changes directories.
// In Rftp, requests to change directories are handled by this class.

public class RftpCurrentDirectory {
private StringBuffer wDir = new StringBuffer("");
private String separator;
private boolean atTop;
private String defaultWorkingDir;
private boolean formatError;
private boolean onWindows = true;

public RftpCurrentDirectory(String wd) 
{
	int mslast = wd.lastIndexOf("\\");
	int unixlast = wd.lastIndexOf("/");
	if (mslast > unixlast) separator = "\\";
    else {
            separator = "/";
            onWindows = false;
         }
	// if the directory pathname ends in "\.", strip off the '.'
	int endcheck = wd.lastIndexOf(separator + ".");
	String wdchecked;
	if (endcheck == (wd.length() - 2))
	wdchecked = wd.substring(0,wd.length() - 1);
	else wdchecked = wd;
	// verify that the pathname ends in a separator
	endcheck = wdchecked.lastIndexOf(separator);
  //System.out.println("checking: " + wdchecked + " endcheck: " + Integer.toString(endcheck) + " length: " + Integer.toString(wdchecked.length()) );
	if (endcheck == wdchecked.length() - 1) wDir.append(wdchecked);
	else wDir.append(wdchecked).append(separator);
	defaultWorkingDir = wDir.toString();
  //System.out.println("wDir: " + wDir.toString() + " default: " + defaultWorkingDir);
}   //end public RftpCurrentDirectory() constructor
//*****************
//*****************
public String adjustRelativePathname(String pathnm){return performAdjust(pathnm);}
//*****************
//*****************
public String adjustRelativePathname(String cmd, int tk)
{
	 StringBuffer newCommand = new StringBuffer(100);
	 StringTokenizer st = new StringTokenizer(cmd);
	 int n = st.countTokens();
	 int i;
	 if (tk > n) return cmd;   // no pathname in command
	 for (i = 1; i < tk ; i++) newCommand.append(st.nextToken()).append(" ");  
	 newCommand.append(performAdjust(st.nextToken())).append(" ");   // the changed pathname
	 for (i++;i <= n; i++) newCommand.append(st.nextToken()).append(" ");  

	 return newCommand.toString();
}
public String changeDirectory(String cmd) 
{
  formatError = false;
  if (cmd.startsWith("CDUP")) goUpOneDir();
  if (cmd.startsWith("CWD"))  changeWorkingDir(cmd);
  if (cmd.startsWith("CD ") || formatError || (cmd.equals("CD"))) setWorkingDir(defaultWorkingDir);
  // System.out.println("Returning wDir: " + wDir.toString());
  return wDir.toString();
}
//*****************
private void changeWorkingDir(String cmd)
{
	 StringTokenizer st = new StringTokenizer(cmd);
	 int n = st.countTokens();
	 String skipover = st.nextToken();  // the CWD command
	 if (n > 1)  //  should be - directory name given with CWD command
	 {
	 // need to handle both relative and absolute pathnames
	 String pathName = st.nextToken();
	 if (!pathName.startsWith("..")) 
	 { //ABSOLUTE PATH
	   // verify that the pathname ends in a separator
		 int endcheck = pathName.lastIndexOf(separator);
		 if (endcheck == pathName.length() - 1) setWorkingDir(pathName);
	     else setWorkingDir(pathName + separator);
	 }
	 else
	 { // RELATIVE PATH
     //System.out.println("Relative path");
	   StringTokenizer p = new StringTokenizer(pathName,separator);
	   int dirs = p.countTokens();
		 boolean stillGoingUpTree = true;
	   StringBuffer next = new StringBuffer(25);
	   for (int cnt = 0; cnt < dirs; cnt++)
	   {
	     next.setLength(0);
	     next.append(p.nextToken());
		   //System.out.println("next: " + next.toString());

	     if (stillGoingUpTree)
	     {
		  if (next.toString().startsWith(".."))
		  {
		    for (int dot = 2; dot < next.length(); dot++)
			if (next.charAt(dot) != '.') formatError = true;
		    if (!formatError) 
			for (int dot = 1; dot < next.length(); dot++) goUpOneDir();
		  }
		  else
		  {
		    stillGoingUpTree = false;
		    wDir.append(next).append(separator);
		  }
	     }
	     
	   }
	 }
	 }
}
//*****************
//*****************
private boolean fullPath(String pathnm) 
{   //MicroSoft separator is \   unix separator is /
	if ((separator.equals("\\")) && (pathnm.indexOf(":\\") == 1)) return true;
	else if ((separator.equals("/")) && (pathnm.indexOf("/") == 0)) return true;    
		   else return false;
}
//*****************
//*****************
public String getWorkingDir(){ return wDir.toString();}
private void goUpOneDir()
{
	  // lastSlash is the SECOND to last slash, as the wDir always ends in a slash
	String wd = wDir.toString();
	int lastSlash = wd.lastIndexOf(separator,wd.length() - 2);
  //System.out.println("goUpOneDir: " + wd + " lastslash: " + Integer.toString(lastSlash));
	if (lastSlash >= 0)   // should always be >= 0
	  {
	  String t = wd.substring(0,lastSlash + 1);
	  setWorkingDir(t);
	  }
   //System.out.println("wDir: " + wDir.toString());
}
//*****************
//*****************
private boolean justFilename(String pathnm) 
{
   if (pathnm.indexOf(separator) >= 0) return false;
	else //if it contains only '.'s then it's relative pathname
	for (int i = 0; i < pathnm.length(); i++)
	  if (pathnm.charAt(i) != '.') return true;
   return false;
}
//*******************************************************************************************
//*******************************************************************************************
public static void main(String[] args) 
{
	RftpCurrentDirectory client = new RftpCurrentDirectory(args[0]);
}   //end public static void main(String[] args) 
//*****************
//*****************
private String performAdjust(String pathnm)
{ 
  // if it is just the filename and workingDir is the default, just return it
  // if it is just the filename & workingDir isn't the default, return appended filename
  // if its a full pathname, just return it 
  // if its a relative pathname, calculate and return actual pathname
  
  if (justFilename(pathnm))
  {
	if ( defaultWorkingDir.equals(wDir.toString())) return pathnm;
	 else
	 {
	   StringBuffer retPath = new StringBuffer(wDir.toString());
 	   return retPath.append(pathnm).toString();
	 }
  }
  if (fullPath(pathnm)) return pathnm;
   else 
   {
	    StringBuffer retrelPath = new StringBuffer(wDir.toString());
	    return retrelPath.append(pathnm).toString();
   }
}
//*****************
public void setWorkingDir(String s)
{
  wDir.setLength(0);
  wDir.append(s);
}
} //end class RftpCurrentDirectory