//Title:       RFTP DataClient
///Version:    1.0
//Copyright:   1999
//Author:      Mary Grzywa
//Company:     Illinois State University Graduate Project
//Description: Resumable File Transfer Protocol
package rftp;

import java.util.*;
import java.util.Vector.*;
import java.util.Date.*;
import java.lang.*;
import java.lang.String.*;
import java.io.*;
import java.net.*;

// Performs file transfers and data transfers over the data socket
public class RftpDataClient {

private static final int SERVER_DATA_PORT = 2302;
private static final int client_data_port = 2405;
private static final boolean AUTOFLUSH = true;
private static final int MAX_CLIENTS = 50;
private ServerSocket listenDataSocket = null;

private Socket dataSocket = null;
private BufferedReader charReceive = null;
private PrintWriter charSend = null;
private BufferedInputStream byteReceive = null;
private BufferedOutputStream byteSend = null;
private boolean OPENDATA = false;
private boolean DEBUG = false;
private long TotalBytesToTransfer = 0;
private long BytesTransferred = 0;
//public long transferredBytes = 0;

/**
 * Constructor
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public RftpDataClient() {
	super();
	initialize();
}
//*******************************************************************************************
//*******************************************************************************************

public boolean acceptDataConnection(String streamType)
{ // System.out.println("in acceptDataConnection");
	if (listenDataSocket == null) listenForServer();
   try
   {
	 //  System.out.println("in acceptDataConnection listendatasocket");
	dataSocket = listenDataSocket.accept();
	  if (streamType.indexOf("CHARACTER") >= 0)
 	{
	   charReceive = new BufferedReader(new InputStreamReader(dataSocket.getInputStream()));
	   charSend = new PrintWriter(dataSocket.getOutputStream(),AUTOFLUSH);
	}
	else if (streamType.indexOf("FILE") >= 0)
	{
	   byteReceive = new BufferedInputStream(dataSocket.getInputStream());
	   byteSend = new BufferedOutputStream(dataSocket.getOutputStream(),64000);
	}
   }
   catch (IOException excpt) 
   {
	  System.out.println("I/O Exception while establishing data connection: " + excpt);
	  return false;
   }
   OPENDATA = true;
   return true;
}
//*******************************************************************************************
//*******************************************************************************************


public void closeDataConnection()
{
	try
	{ 
	  if (DEBUG) System.out.println("DEBUG: Closing data connections.  " );
	  if (charSend != null) charSend.close();
	  if (charReceive != null) charReceive.close();
	  if (byteSend != null) byteSend.close();
	  if (byteReceive != null) byteReceive.close();
	  if (dataSocket != null)  dataSocket.close();
	  OPENDATA = false;
	} catch (IOException excpt) {System.out.println("Failed I/O to server while closing data connection: " + excpt);}


}
//*******************************************************************************************
//*******************************************************************************************

public String dataStringIn()
{
  String dataInput = null;
  char b[] = new char[64000];
  int j = 0;

  try 
  {
	int dataSize = charReceive.read(b,0,64000);
	dataInput = new String(b,0,dataSize);
  }
  catch (IOException excpt) 
	{
	System.out.println("I/O Exception in dataStringIn: " + excpt);
	System.exit(1);
	}
  catch (StringIndexOutOfBoundsException eeee)
  // this can occur if the read is performed before data has arrived  - wait & try again
  {
	try 
	{
	for (int i = 1;i < 3000000; i++ );
	int dataSize2 = charReceive.read(b,0,64000);
	dataInput = new String(b,0,dataSize2);
	}
	catch (IOException excpt) 
	{
	System.out.println("I/O Exception in dataStringIn: " + excpt);
	System.exit(1);
	}
	catch (StringIndexOutOfBoundsException eee) {}

  }
   if (DEBUG) System.out.println("DEBUG: Printing dataInput line ");   
   return dataInput;
}
/**
 * Called whenever the part throws an exception.
 * @param exception java.lang.Throwable
 */
private void handleException(Throwable exception) {

	/* Uncomment the following lines to print uncaught exceptions to stdout */
	// System.out.println("--------- UNCAUGHT EXCEPTION ---------");
	// exception.printStackTrace(System.out);
}
/**
 * Initialize the class.
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void initialize() {
	// user code begin {1}
	// user code end
	// user code begin {2}
	// user code end
}
//*******************************************************************************************
//*******************************************************************************************

public boolean isOpen()
{
 if (dataSocket == null)  return false;  else return true;
}
//*******************************************************************************************
//*******************************************************************************************

public void listenForServer()
{
	try
	{
	   // create listen socket
	   listenDataSocket = new ServerSocket(client_data_port,MAX_CLIENTS);
	} catch(IOException excpt) 
		{
	      System.out.println("Rftp IOException while listening for server:" + excpt);
			System.exit(1);
		}
}
/**
 * main entrypoint - starts the part when it is run as an application
 * @param args java.lang.String[]
 */
public static void main(java.lang.String[] args) {
	try {
		RftpDataClient aRftpDataClient;
		aRftpDataClient = new RftpDataClient();
	} catch (Throwable exception) {
		System.err.println("Exception occurred in main() of java.lang.Object");
		exception.printStackTrace(System.out);
	}
}
//*******************************************************************************************
//*******************************************************************************************
//public long getTransferredBytes() {return transferredBytes;}
//*******************************************************************************************
//*******************************************************************************************
public void setDEBUG(boolean setTo)
{
   DEBUG = setTo;
}

//*******************************************************************************************
//*******************************************************************************************
public void transferFileIn(File f, int pos)
{
  byte[] buf = new byte[64000];
  int amtRead;
  BufferedOutputStream fileOut = null;
 // transferredBytes = 0;
  try 
  {
	
	  RandomAccessFile fo = new RandomAccessFile(f,"rw");  
	int pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("filepointer: " + Integer.toString(pointAt));
	if (pos > 0) fo.seek((long) pos);
	pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("filepointer: " + Integer.toString(pointAt));
	
	  if (DEBUG) System.out.println("DEBUG: In transferFileIn ");   
	fileOut = new BufferedOutputStream (new FileOutputStream(fo.getFD()), 64000);
	  if (DEBUG) System.out.println("DEBUG: Created BufferedOutputStream");
	amtRead = 1;
  BytesTransferred = 0;
 	while (amtRead > 0)
	{
	  amtRead = byteReceive.read(buf,0,64000);
		// too much printing to see what's going on - commented line out
		//  if (DEBUG) System.out.println("DEBUG: reading input stream, writing to file. amtRead:" + Integer.toString(amtRead));   
	  if (amtRead > 0) fileOut.write(buf,0,amtRead);
 //   transferredBytes = transferredBytes + (long) amtRead;
    BytesTransferred = BytesTransferred + amtRead;
	}
  } 
  catch (IOException eeeee) 
  {  // normal end to file transfer - data stream closed - also occurs upon read/write fail
  }
  try {fileOut.close();} catch (IOException eeeee) { }
  BytesTransferred = (long) 0;
  TotalBytesToTransfer = (long) 0;
}
//*******************************************************************************************
//*******************************************************************************************

public void transferFileOut(File outFile,int startPosition)
{
  byte[] buf = new byte[64000];
  int amtRead = 0;
  long transferSize;
  int sizeOfRead;
 // transferredBytes = 0;
  try 
  {
	RandomAccessFile fo = new RandomAccessFile(outFile,"rw");  
	int pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("filepointer: " + Integer.toString(pointAt));
	if (startPosition > 0) fo.seek((long) startPosition);
	pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("filepointer: " + Integer.toString(pointAt));
	
	transferSize = outFile.length() - (long)startPosition;
  TotalBytesToTransfer = transferSize;
	BufferedInputStream fileIn = new BufferedInputStream (new FileInputStream(fo.getFD()), 64000);
 	while ((amtRead < transferSize) && OPENDATA)
	{
  	  if ( (transferSize - amtRead) >= 64000) sizeOfRead = 64000;
		 else sizeOfRead = (int) (transferSize - amtRead);
	  int tempint = sizeOfRead;
		// too much printing to see what's going on - commented line out
	  // if (DEBUG) System.out.println("DEBUG File transfer: Amt Read so far: " +
	  //	Integer.toString(amtRead) + " sizeofRead:" + Integer.toString(tempint));
	  fileIn.read(buf,0,sizeOfRead);
	  byteSend.write(buf,0,sizeOfRead);
	  amtRead = sizeOfRead + amtRead;
    BytesTransferred = amtRead;
 //   transferredBytes = (long) amtRead;
	}
	  fileIn.close();
    BytesTransferred = (long) 0;
    TotalBytesToTransfer = (long) 0;
  } 
  catch (IOException eeeee) 
  {  // also occurs upon read/write fail
  }
}

public int getPercentTransferred() {
if (TotalBytesToTransfer == 0  ) return 0;
float percent;
percent  = ( ((float)BytesTransferred / (float) TotalBytesToTransfer) * 100);
//System.out.println( TotalBytesToTransfer + " " + BytesTransferred + " " + percent);
return  (int) percent;
}

public void setTotalBytesToTransfer(long ll) {
TotalBytesToTransfer = ll;
}
}  //end class RftpDataClient
