//package rftp;

import java.util.*;
import java.util.Vector.*;
import java.util.Date.*;
import java.lang.*;
import java.lang.String.*;
import java.io.*;
import java.net.*;


public class RftpDataServer 
{

private static final int MAX_CLIENTS = 50;
private static final boolean AUTOFLUSH = true;
private InetAddress server;
private String IPaddress;
private InetAddress clientIP;
private int client_data_port = 2405;
private Socket dataSocket = null;
private BufferedReader charReceive = null;
private PrintWriter charSend = null;
private BufferedInputStream byteReceive = null;
private BufferedOutputStream byteSend = null;
private boolean OPENDATA = false;
private boolean DEBUG = false;
private int clientId = 0;
//*******************************************************************************************
//*******************************************************************************************
public RftpDataServer(InetAddress client,int clientNumber)
{
clientIP = client;
clientId = clientNumber;
}
//*******************************************************************************************
//*******************************************************************************************

public boolean openDataConnection(String streamType)
{   /*if (DEBUG) */ System.out.println("DEBUG Client " + clientIP + ": Opening Data Connection ");
    //connect the data link
    if (!OPENDATA) try
    {   
       dataSocket = new Socket (clientIP, client_data_port);
	/* if (DEBUG)*/ System.out.println("DEBUG Client " + clientId + ": Socket: " + dataSocket.toString());

      if (streamType.equals("CHARACTER DATA"))
 	{
       charReceive = new BufferedReader(new InputStreamReader(dataSocket.getInputStream()));
	 /* if (DEBUG)*/ System.out.println("DEBUG Client " + clientId + ": Data Buffered Reader: " + charReceive.toString());

       charSend = new PrintWriter(dataSocket.getOutputStream(),AUTOFLUSH);
	 if (DEBUG) System.out.println("DEBUG Client " + clientId + ": Data PrintWriter: " + charSend.toString());
	}
	else if (streamType.equals("FILE TRANSFER"))
	{
	   byteReceive = new BufferedInputStream(dataSocket.getInputStream());
	   byteSend = new BufferedOutputStream(dataSocket.getOutputStream(),64000);
	}

       OPENDATA = true;
	return true; 
    }
    catch (UnknownHostException excpt) 
    {  
	// if (DEBUG) 
	  System.out.println("DEBUG Client " + clientId + ": UnknownHost Exception: " + excpt);  	 
	  closeDataConnection();
        return false;
    }
    catch (IOException excpt) 
    {  
	 //if (DEBUG) 
	  System.out.println("DEBUG Client " + clientId + ": UnknownHost Exception: " + excpt);  	 
        closeDataConnection();
	  return false;
    }
    else 
    {
	/* if (DEBUG)*/ System.out.println("DEBUG Client " + clientId + ": Returning true with an open data connection.");
	return true;
    }
}  // end openDataConnection()

//*******************************************************************************************
//*******************************************************************************************

public void closeDataConnection()
{
        
    try
    {
	if (DEBUG) System.out.println("DEBUG Client " + clientId + ": Closing data connection." );
      Thread.sleep(1505);
    } 
    catch (InterruptedException  eee) {}

    if (OPENDATA) try
    {
	if (charSend != null) charSend.close();
      if (charReceive != null) charReceive.close();
	if (byteSend != null) byteSend.close();
      if (byteReceive != null) byteReceive.close();
      if (dataSocket != null)  dataSocket.close();
    }
    catch (IOException excpt) 
	  {if (DEBUG) 
            System.out.println("426 Connection closed; Exception during closing." + excpt);
	   OPENDATA = false;
        }
	   
    OPENDATA = false;
}

//*******************************************************************************************
//*******************************************************************************************
public String getDataString()
{
   String dataInput = null;
   try
    {
       dataInput = charReceive.readLine();
    } 
   catch (IOException excpt) 
    {
	if (DEBUG) System.out.println("I/O Exception While contacting server: " + excpt);
	System.exit(1);
    }
   if (DEBUG) System.out.println(dataInput);
   return dataInput;
}



//*******************************************************************************************
//*******************************************************************************************
public void sendData(String h)
{
   if (DEBUG) System.out.println("DEBUG Client " + clientId + ": Sending the data over data connection.");
  if (DEBUG) System.out.println(h);
   charSend.print(h);
   charSend.flush();
}
//*******************************************************************************************
//*******************************************************************************************

public void transferFileOut(File outFile,int startPosition)
{
  byte[] buf = new byte[64000];
  int amtRead = 0;
  long fileSize;
  int transferAmt;
  int sizeOfRead;
  try 
  {
	RandomAccessFile fo = new RandomAccessFile(outFile,"rw");  
	int pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("DEBUG Client " + clientId + " filepointer: " + Integer.toString(pointAt));
	if (startPosition > 0) fo.seek((long) startPosition);
	pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("DEBUG Client " + clientId + " filepointer: " + Integer.toString(pointAt));

	fileSize = outFile.length();
      transferAmt = (int) fileSize - startPosition;
	BufferedInputStream fileIn = new BufferedInputStream (new FileInputStream(fo.getFD()), 64000);
 	while ((amtRead < transferAmt) && OPENDATA)
	{
  	  if ( (transferAmt - amtRead) >= 64000) sizeOfRead = 64000;
		 else sizeOfRead = (int) (transferAmt - amtRead);
	  fileIn.read(buf,0,sizeOfRead);
	  byteSend.write(buf,0,sizeOfRead);
	  amtRead = sizeOfRead + amtRead;
	}
      if (DEBUG) System.out.println("DEBUG Client " + clientId + ": File transfer: Amt Read : " + 
		Integer.toString(amtRead)  );
	fileIn.close();
  } 
  catch (IOException eeeee) 
  {  // also occurs upon read/write fail
  }
}

//*******************************************************************************************
//*******************************************************************************************

public void transferFileIn(File outFile, int startPosition)
{
  byte[] buf = new byte[64000];
  int amtRead;
  BufferedOutputStream fileOut = null;
  try 
  {
      RandomAccessFile fo = new RandomAccessFile(outFile,"rw");  
	int pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("DEBUG Client " + clientId + " filepointer: " + Integer.toString(pointAt));
	if (startPosition > 0) fo.seek((long) startPosition);
	pointAt = (int) fo.getFilePointer();
	if (DEBUG) System.out.println("DEBUG Client " + clientId + " filepointer: " + Integer.toString(pointAt));

	if (DEBUG) System.out.println("DEBUG: In transferFileIn ");   
	fileOut = new BufferedOutputStream (new FileOutputStream(fo.getFD()), 64000);
      if (DEBUG) System.out.println("DEBUG: Created BufferedOutputStream");
	amtRead = 1;   
 	while (amtRead > 0)
	{
	  amtRead = byteReceive.read(buf,0,64000);
        if (amtRead > 0) fileOut.write(buf,0,amtRead);
	}
	if (DEBUG) System.out.println("DEBUG Client " + clientId + " read from input stream, wrote to file. amtRead:" + Integer.toString(amtRead));   
  } 
  catch (IOException eeeee) 
  {  // normal end to file transfer - data stream closed - also occurs upon read/write fail
  }
  try {fileOut.close();} catch (IOException eeeee) { }
}

//*******************************************************************************************
//*******************************************************************************************
public void setDEBUG(boolean h)
{
   DEBUG = h;
}
//*******************************************************************************************
//*******************************************************************************************
public String connectionStatus()
{
  if (OPENDATA) return ("Open"); else return ("Closed");
}

}  //end class RftpDataServer